@@ -5,10 +5,13 @@
  * 1999-04-22  V 1.1.1 
  * 1999-07-01  V 1.2.0 beta
  * 1999-10-22  V 1.2.0 final
+ * 2000-05-10  V 1.3.0 alpha
+ * 2000-10-24  V 1.3.0 final
+ * 2002-01-03  V 1.3.1
  *
  * NOTE: Edit this file with tabstop=4 !
  *
- * Copyright 1996-1999 by Gerhard Buergmann
+ * Copyright 1996-2002 by Gerhard Buergmann
  * Gerhard.Buergmann@altavista.net
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -34,7 +37,7 @@
 #endif
 
 
-char	*copyright  = "Copyright (C) 1996-1999 by Gerhard Buergmann";
+char	*copyright  = "Copyright (C) 1996-2002 by Gerhard Buergmann";
 
 jmp_buf	env;        /* context for `longjmp' function   */
 
@@ -50,13 +53,16 @@ PTR		spos;
 char	*name = NULL;
 char	*shell;
 char	string[MAXCMD];
-char	cmdstr[MAXCMD + 1];
+char	cmdstr[MAXCMD + 1] = "";
 FILE	*Ausgabe_Datei;
 int		edits = 0;
 int		AnzAdd, Anzahl, Anzahl3;
 off_t	filesize, memsize, undosize;
 long	precount = -1;	/* number preceding command */
 
+int		block_flag = 0;
+off_t	block_begin, block_end, block_size;
+
 
 char	**files;		/* list of input files */
 int		numfiles;		/* number of input files */
@@ -64,7 +70,6 @@ int		curfile;		/* number of the current file */
 
 int		arrnum = 0;
 char	numarr[64];		/* string for collecting number */
-char	progname[15];
 char	rep_buf[BUFFER];
 
 PTR		current;
@@ -81,6 +86,7 @@ char	addr_form[15];
 
 char	*nobytes = "No bytes@in the buffer";
 
+static	char	progname[8];
 static	char	line[MAXCMD];
 static	int		mark;
 static	int		scrolly;
@@ -90,7 +96,8 @@ static	int		wrstat = 1;
 void
 usage()
 {
-	fprintf(stderr, "Usage: %s [-R] [-c command] [-f script] file ...\n", progname);
+	fprintf(stderr, "Usage: %s [-R] [-c cmd | +cmd] [-f script]\n\
+       [-b begin] [-e end] [-s size] file ...\n", progname);
 	exit(1);
 }
 
@@ -113,8 +120,8 @@ main(argc, argv)
 #endif
 	poi = strrchr(argv[0], DELIM);
 
-	if (poi) strcpy(progname, ++poi);
-		else strcpy(progname, argv[0]);
+	if (poi) strncpy(progname, ++poi, 7);
+		else strncpy(progname, argv[0], 7);
 	strtok(progname, ".");
 
 	if (!strcasecmp(progname, "bview")) {
@@ -134,8 +141,7 @@ main(argc, argv)
 				if (argv[n + 1] == NULL) {
 					usage();
 				} else {
-					sprintf(string, "%s\n", argv[++n]);
-					stuffin(string);
+					strcpy(cmdstr, argv[++n]);
 				}
 			} else if (argv[n][1] == 'f') {
 				if (argv[n + 1] == NULL || argv[n + 1][0] == '-') {
@@ -143,6 +149,27 @@ main(argc, argv)
 				} else {
 					script = ++n;
 				}
+			} else if (argv[n][1] == 'b') {
+				if (argv[n + 1] == NULL || argv[n + 1][0] == '-') {
+					usage();
+				} else {
+					block_begin = calc_size(argv[++n]);
+					block_flag |= 1;
+				}
+			} else if (argv[n][1] == 'e') {
+				if (argv[n + 1] == NULL || argv[n + 1][0] == '-') {
+					usage();
+				} else {
+					block_end = calc_size(argv[++n]);
+					block_flag |= 2;
+				}
+			} else if (argv[n][1] == 's') {
+				if (argv[n + 1] == NULL || argv[n + 1][0] == '-') {
+					usage();
+				} else {
+					block_size = calc_size(argv[++n]);
+					block_flag |= 4;
+				}
 			} else if (argv[n][1] == 'w') {
 				if (argv[n][2] == '\0') {
 					usage();
@@ -153,12 +180,12 @@ main(argc, argv)
 			} else usage();
 			n++;
 			break;
-		case '+':			/* +/pat */
-			if (argv[n][1] == '/') {
-					sprintf(string, "%s\n", &argv[n][1]);
-					stuffin(string);
-			} else
-				usage();
+		case '+':			/* +cmd */
+			if (argv[n][1] == '\0') {
+				strcpy(cmdstr, "$");
+			} else {
+				strcpy(cmdstr, &argv[n][1]);
+			}
 			n++;
 			break;
 		default:			/* must be a file name */
@@ -169,7 +196,33 @@ main(argc, argv)
 			break;
 		}
 	}
-	if (numfiles > 1) printf("%d files to edit\n", numfiles);
+	switch (block_flag) {
+	case 2:
+		block_begin = 0;
+	case 1|2:
+		block_size = block_end - block_begin + 1;
+		break;
+	case 4:
+		block_begin = 0;
+	case 1|4:
+		block_end = block_begin + block_size - 1;
+		break;
+	case 2|4:
+		block_begin = block_end + 1 - block_size;
+		break;
+	case 1|2|4:
+		if (block_end - block_begin != block_size + 1) {
+			fprintf(stderr, "Ambigous block data\n");
+			exit(1);
+		}
+		break;
+	}
+
+	if (block_flag && !numfiles) {
+		fprintf(stderr, "Cannot read a range of a nonexisting file\n");
+		exit(1);
+	}
+	if (numfiles > 1) fprintf(stderr, "%d files to edit\n", numfiles);
 	curfile = 0;
 
 	/****** Initialisation of curses ******/
@@ -207,9 +260,15 @@ main(argc, argv)
 
 	bvi_init(argv[0]);
 	params[P_TT].svalue = terminal;
+	if (block_flag && (P(P_MM) == TRUE)) {
+		P(P_MM) = FALSE;
+		params[P_TT].flags |= P_CHANGED;
+	}
 	if (script > -1)
 		read_rc(argv[script]);
-
+	if (*cmdstr != '\0')
+		docmdline(cmdstr);
+	
 	/* main loop */
 	do {
 		setjmp(env);
@@ -271,7 +330,7 @@ main(argc, argv)
 					if (loc == HEX) x = AnzAdd;
 						else	x = AnzAdd + Anzahl3;
 					break;
-		case CTRL('H'):
+		case BVICTRL('H'):
 		case KEY_BACKSPACE:
 		case KEY_LEFT:
 		case 'h':	do {
@@ -286,7 +345,10 @@ main(argc, argv)
 		case ' ':
 		case KEY_RIGHT:
 		case 'l':	do {
+						/*
 						if (x < (Anzahl3 + 6))  x += 3;
+						*/
+						if (x < (Anzahl3 + AnzAdd - 2))  x += 3;
 							else if (x > (Anzahl3 + 3)
 								&& x < (Anzahl3 + AnzAdd - 1 + Anzahl))
 									x++;
@@ -305,8 +367,8 @@ main(argc, argv)
 		case CR:	if (loc == HEX) x = AnzAdd;
 						else		x = AnzAdd + Anzahl3;
 		case 'j':
-		case CTRL('J'):
-		case CTRL('N'):
+		case BVICTRL('J'):
+		case BVICTRL('N'):
 		case KEY_DOWN:
 					do {
 						if ((PTR)((pagepos + (y + 1) * Anzahl)) > maxpos) break;
@@ -328,25 +390,25 @@ main(argc, argv)
 					if (strlen(cmdstr))
 						docmdline(cmdstr);
 					break;
-		case CTRL('B'):
+		case BVICTRL('B'):
 		case KEY_PPAGE: 	/**** Previous Page ****/
 					if (mem <= (PTR)(pagepos - screen))	pagepos -= screen;
 						else	pagepos = mem;
 					repaint();
 					break;
-		case CTRL('D'):
+		case BVICTRL('D'):
 					if (precount > 1) scrolly = precount;
 					scrolldown(scrolly);
 					break;
-		case CTRL('U'):
+		case BVICTRL('U'):
 					if (precount > 1) scrolly = precount;
 					scrollup(scrolly);
 					break;
-		case CTRL('E'):
+		case BVICTRL('E'):
 					if (y > 0) y--;
 					scrolldown(1);
 					break;
-		case CTRL('F'):
+		case BVICTRL('F'):
 		case KEY_NPAGE: 	/**** Next Page *****/
 					if (maxpos >= (PTR)(pagepos + screen))  {
 						pagepos += screen;
@@ -358,14 +420,14 @@ main(argc, argv)
 						repaint();
 					}
 					break;
-		case CTRL('G'):
+		case BVICTRL('G'):
 					fileinfo(name);
 					wrstat = 0;
 					break;
-		case CTRL('L'):   	/*** REDRAW SCREEN ***/
+		case BVICTRL('L'):   	/*** REDRAW SCREEN ***/
 					new_screen();
 					break;
-		case CTRL('Y'):
+		case BVICTRL('Y'):
 					if (y < maxy) y++;
 					scrollup(1);
 					break;
@@ -482,6 +544,7 @@ main(argc, argv)
 					break;
 		case 'r':
 		case 'R':   if (filesize == 0L) break;
+					if (precount < 1) precount = 1;
 					sprintf(rep_buf, "%ld%c", precount, ch);
 					undo_count = edit(ch);
 					lflag++;
@@ -538,7 +601,7 @@ main(argc, argv)
 /* undo does not work correctly !!! */
 				case 's':
 					sprintf(rep_buf, "%lds", precount);
-					if (do_delete(precount, current)) break;
+					if (do_delete((off_t)precount, current)) break;
 					precount = 1;
 					undo_count = edit('i');
 					lflag++;
@@ -559,9 +622,9 @@ main(argc, argv)
 				case 'd':
 					count = range(ch);
 					if (count > 0)
-						do_delete(count, current);
+						do_delete((off_t)count, current);
 					else if (count < 0)
-						do_back(-count, current);
+						do_back(-(off_t)count, current);
 					if (ch == 'c') {
 						precount = 1;
 						undo_count = edit('i');
@@ -575,11 +638,11 @@ main(argc, argv)
 					break;
 				case 'x':
 					sprintf(rep_buf, "%ldx", precount);
-					do_delete(precount, current);
+					do_delete((off_t)precount, current);
 					break;
 				case 'X':
 					sprintf(rep_buf, "%ldX", precount);
-					do_back(precount, current);
+					do_back((off_t)precount, current);
 					break;
 				default:
 					flushinp();
@@ -587,11 +650,21 @@ main(argc, argv)
 				}
 			} else {
 				switch (ch) {
+				case 'x':
+					if (precount < 1) precount = 1;
+					if ((off_t)precount + current == maxpos) {
+						sprintf(rep_buf, "%ldx", precount);
+						do_delete((off_t)precount, current);
+					} else {
+						movebyte();
+						flushinp();
+						beep();
+					}
+					break;
 				case 'd':
 				case 'i':
 				case 'I':
 				case 'p':
-				case 'x':
 				case 'X':
 					movebyte();
 				default:
@@ -605,6 +678,32 @@ main(argc, argv)
 }
 
 
+off_t
+calc_size(arg)
+	char	*arg;
+{
+	long	val;
+	char	*poi;
+
+	if (*arg == '0') {
+		val = strtol(arg, &poi, 16);
+	} else {
+		val = strtol(arg, &poi, 10);
+	}
+	switch (*poi) {
+	case 'k':
+	case 'K':	val *= 1024;
+			 	break;
+	case 'm':
+	case 'M':	val *= 1048576;
+			 	break;
+	case '\0':	break;
+	default:	usage();
+	}
+	return (off_t)val;
+}
+
+
 void
 trunc_cur()
 {
@@ -654,9 +753,9 @@ do_tilde(count)
 		return;
 	memcpy(undo_buf, current, undo_count);
 	while (count--) {
-		if (isupper(*current)) *current = tolower(*current);
-			else if (islower(*current))
-				*current = toupper(*current);
+		if (isupper((int)(*current))) *current = tolower((int)(*current));
+			else if (islower((int)(*current)))
+				*current = toupper((int)(*current));
 		current++;
 		cur_forw(0);
 	}
